/* gvSIG. Geographic Information System of the Valencian Government
 *
 * Copyright (C) 2007-2008 Infrastructures and Transports Department
 * of the Valencian Government (CIT)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 */

package org.gvsig.fortunecookies.main;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;

import org.gvsig.fortunecookies.FortuneCookieLocator;
import org.gvsig.fortunecookies.FortuneCookieManager;
import org.gvsig.fortunecookies.FortuneCookieService;
import org.gvsig.fortunecookies.exception.FortuneCookieMessageException;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.library.impl.DefaultLibrariesInitializer;
import org.gvsig.tools.service.ServiceException;

public class Main implements ComponentListener {
	
	FortuneCookieManager manager;
	
	public static final int MODE_DIALOG = 1;
	public static final int MODE_WINDOW = 2;
	public static final int MODE_TOOL = 3;
	
	public static void main( String args[] ) {
		Main main = new Main();
		main.show();
	}
	
	public void show() {
			
		org.apache.log4j.BasicConfigurator.configure();
		
		new DefaultLibrariesInitializer().fullInitialize();
		manager = FortuneCookieLocator.getManager();
						
		JFrame frame = new JFrame("Fortune Cookie");
		
		// Create the menu bar.
		JMenuBar menuBar = new JMenuBar();

		// Build the menu.
		JMenu menu_file = new JMenu("File");
		JMenuItem menuItem_get = new JMenuItem("Get a fortune cookie");
		menuItem_get.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				showFortuneCookie(manager);
			}
	    });
		menu_file.add(menuItem_get);

	
		JMenuItem menuItem_exit = new JMenuItem("Exit");
		menuItem_exit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
	    });
		menu_file.add(menuItem_exit);
		
		menuBar.add(menu_file);
		
		JToolBar toolBar = new JToolBar();
		
		JButton get = new JButton("Get a fortune cookie");
		get.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {
				showFortuneCookie(manager);			
			}
			
		});
		toolBar.add(get);

		frame.setPreferredSize(new Dimension(160, 100));
		frame.add(menuBar, BorderLayout.NORTH);
		frame.add(toolBar, BorderLayout.LINE_END);
		
        //Display the window.
        frame.pack();
        frame.setVisible(true);
        		
	}
	
	public void showFortuneCookie(FortuneCookieManager manager) {
		try {
			DynObject parameters = manager.createServiceParameters("FortuneCookie.providers.web");
			FortuneCookieService service = (FortuneCookieService) manager.getService(parameters);
			JPanel panel = createFortuneCookieServicePanel(service);
			showWindow(
					panel, 
					"Fortune Cookie", 
					MODE_WINDOW
			);

		} catch (ServiceException e) {
			e.printStackTrace();
		}
	}

	public JPanel createFortuneCookieServicePanel(final FortuneCookieService cookie) {
		final JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.setPreferredSize(new Dimension(550,150));

		JTextArea text = new JTextArea();
		try {
			text.setText(cookie.getMessage());
		} catch (FortuneCookieMessageException e) {
			text.setText(this.getTranslation("Error obtaining message....."));
			e.printStackTrace();
		}
		text.setEditable(false);
		
	    JScrollPane scrollPane = new JScrollPane(text);
	    scrollPane.setPreferredSize(new Dimension(550, 150));

	    JButton accept = new JButton(this.getTranslation("Accept"));
	    JButton info = new JButton(this.getTranslation("Info"));
		
		JPanel optionsPane = new JPanel();
		optionsPane.setLayout(new BoxLayout(optionsPane, BoxLayout.LINE_AXIS));
		optionsPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		optionsPane.add(Box.createHorizontalGlue());
		
		accept.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				panel.setVisible(false);
			}
		});
		info.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				showInfo(cookie);
			}
		});
		
		optionsPane.add(accept);
		optionsPane.add(info);
		optionsPane.add(Box.createRigidArea(new Dimension(10, 0)));
		
		panel.add(scrollPane, BorderLayout.CENTER);
		panel.add(optionsPane, BorderLayout.SOUTH);
		
		return panel;
	}

	private void showInfo(FortuneCookieService cookie) {
		JPanel infoPanel = createFortuneCookieServiceInfoPanel(cookie);
		showWindow(
				infoPanel, 
				this.getTranslation("Information"), 
				MODE_DIALOG
		);
	}

	private JPanel createFortuneCookieServiceInfoPanel(FortuneCookieService cookie) {
		final JPanel panel = new JPanel();
		
		panel.setLayout(new BorderLayout());
		panel.setPreferredSize(new Dimension(550,150));

		JTextArea text = new JTextArea();
		try {
			text.setText("Message: " +cookie.getMessage()+"\nDate: " + cookie.getDate());
		} catch (FortuneCookieMessageException e) {
			text.setText(this.getTranslation("Error obtaining message....."));
			e.printStackTrace();
		}
		text.setEditable(false);
		
	    JScrollPane scrollPane = new JScrollPane(text);
	    scrollPane.setPreferredSize(new Dimension(550, 150));

	    JButton accept = new JButton(this.getTranslation("Accept"));
		
		JPanel optionsPane = new JPanel();
		optionsPane.setLayout(new BoxLayout(optionsPane, BoxLayout.LINE_AXIS));
		optionsPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		optionsPane.add(Box.createHorizontalGlue());
		
		accept.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				panel.setVisible(false);
			}
		});
		optionsPane.add(accept);
		optionsPane.add(Box.createRigidArea(new Dimension(10, 0)));
		
		panel.add(scrollPane, BorderLayout.CENTER);
		panel.add(optionsPane, BorderLayout.SOUTH);
		
		return panel;
	}
	
	public void showWindow(JPanel panel, String title, int mode) {
		JFrame frame;
		
		frame = new JFrame();
		//frame.setAlwaysOnTop(true);
		
		final Dimension s = Toolkit.getDefaultToolkit().getScreenSize();
		frame.setLocation(s.width / 4, s.height / 4);
		
		frame.setLayout(new BorderLayout());
		frame.setTitle(title);
		frame.add(panel, BorderLayout.CENTER);
		
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		frame.setSize(400,400);
		panel.addComponentListener(this);
		
		frame.pack();
		frame.setVisible(true);
	}
			
	public void componentHidden(ComponentEvent arg0) {
		JFrame frame = (JFrame) ((JPanel) arg0.getSource()).getRootPane().getParent();
		frame.setVisible(false);
		frame.dispose();
	}
	public void componentMoved(ComponentEvent arg0) {
		// Do nothing
		
	}
	public void componentResized(ComponentEvent arg0) {
		// Do nothing
		
	}
	public void componentShown(ComponentEvent arg0) {
		// Do nothing
		
	}
	
	public String getTranslation(String string) {

		return string;
	}
} 